#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "pcl711.h"

#define BUFFER_SIZE 1000
float ADbuffer[BUFFER_SIZE];
unsigned DIbuffer[BUFFER_SIZE];
FILE *file;

enum operations {AI, DI, AO, DO} operation;

/*----------------------------------------------------------------------------*/
void Help ()
{
  puts ("ctrl711 -AI <samples/s> <seconds> <file name> [-g=1|2|4|8|16] [-i=<input 0-7>] [-a=<I/O address (default 0x210)>");
  puts ("ctrl711 -DI <samples/s> <seconds> <file name> [-a=<I/O address (default 0x210)>");
  puts ("ctrl711 -AO <raw value> [-a=<I/O address (default 0x210)>");
  puts ("ctrl711 -DO 0|1 [-o=output 0-15] [-a=<I/O address (default 0x210)>");
  puts ("gain list:");
  puts ("x1: [-5,+5]V");
  puts ("x2: [-2.5,+2.5]V");
  puts ("x4: [-1.25,+1.25]V");
  puts ("x8: [-0.625,+0.625]V");
  puts ("x16: [-0.3125,+0.3125]V");
  exit (EXIT_SUCCESS);
}
/*----------------------------------------------------------------------------*/
void ScanParameters (int argc,char *argv[],int *samples,int *seconds,char *FileName,unsigned *gain,int *input,int *addr)
{
  unsigned i, start;
  unsigned DOBit=0, bValue, DAValue;

  if (argc < 3) Help ();

  /* operation */
  if (!strcmp (argv[1],"-AI")) operation=AI;
  else
  if (!strcmp (argv[1],"-DI")) operation=DI;
  else
  if (!strcmp (argv[1],"-AO")) operation=AO;
  else
  if (!strcmp (argv[1],"-DO")) operation=DO;
  else Help ();

  switch (operation)
  {
    case AI :
    case DI :
      if (argc < 5) Help ();
      if ((*samples = atoi(argv[2])) <= 0) Help();
      if ((*seconds = atoi(argv[3])) <= 0) Help();
      strcpy (FileName,argv[4]);
      if (!strchr (FileName,'.')) strcat (FileName,".csv");
      start=5;
      break;
    case AO :
      /* 12-bit analog output */
      if ((DAValue = atoi(argv[2])) >= 4096) Help();
      start=3;
      break;
    case DO :
      if (strcmp(argv[2],"0") && strcmp(argv[2],"1")) Help();
      bValue = !strcmp(argv[2],"0") ? FALSE : TRUE;
      start=3;
      break;
  } 

  /* default I/O address */
  *addr=0x210;

  for (i=start; i < argc; i++)
  {
    /* gain */
    if (operation==AI && !strncmp (argv[i],"-g=",3))
    {
       if (((*gain = atoi(argv[i]+3)) != 1) &&
	   (*gain != 2) &&
	   (*gain != 4) &&
	   (*gain != 8) &&
	   (*gain != 16)) Help();
    }
    else

    /* analog input number */
    if (operation==AI && !strncmp (argv[i],"-i=",3))
    {
       if (((*input = atoi(argv[i]+3)) < 0) || (*input > 7)) Help();
    }
    else

    /* digital output number */
    if (operation==DO && !strncmp (argv[i],"-o=",3))
    {
       if (((DOBit = atoi(argv[i]+3)) < 0) || (DOBit > 15)) Help();
    }
    else

    /* I/O address */
    if (!strncmp (argv[i],"-a=",3))
    {
       sscanf (argv[i]+3,"%X",addr);
       if (*addr <= 0) Help();
    }

    else Help ();
  }

  /* Initialize card */
  InitCard(*addr);
  printf ("I/O address: 0x%X\n",*addr);

  if (operation==AO)
  {
     Write_DA (DAValue);
     printf ("Analog output=%u (%.4f volts with JP1 in factory default)\n",DAValue,5*DAValue/4095.0);
     exit (EXIT_SUCCESS);
  }
  else
  if (operation==DO)
  {
     Write_DO (DOBit,bValue);
     printf ("Digital output #%u=%u\n",DOBit,bValue);
     exit (EXIT_SUCCESS);
  }
}
/*----------------------------------------------------------------------------*/
void WriteBuffer (unsigned i,enum operations operation)
{
  unsigned j;
  int k;

  for (j = 0; j < i; j++)
    if (operation == AI)
       fprintf (file,"%.4f\n",ADbuffer[j]);
    else
    {
       for (k=15; k>=0; k--)
       {
         fprintf (file,"%s",DIbuffer[j] & (((unsigned)0x0001)<<k) ? "1" : "0");
         if (k%4==0) fprintf (file," ");
       }
       fprintf (file,"\n");
    }
}
/*----------------------------------------------------------------------------*/
void main(int argc,char *argv[])
{
  unsigned i;
  unsigned gain = 1;
  int samples, seconds, input = 0, addr;
  /* total samples and current samples */
  float fTotalSamples, fCurrentSamples;
  float fMicroWait;
  char FileName[256];

  ScanParameters (argc,argv,&samples,&seconds,FileName,&gain,&input,&addr);

  fTotalSamples=((float)samples)*seconds;
  printf ("I go to get %u*%u=%.0f %s samples in %u seconds\n",samples,seconds,fTotalSamples,operation==AI ? "analog" : "digital",seconds);

  printf ("Output file name: %s\n",FileName);
  if (operation==AI)
  {
    switch (gain)
    {
      case 1 :
        puts ("gain: x1 [-5,+5]V");
        break;
      case 2 :
        puts ("gain: x2 [-2.5,+2.5]V");
        break;
      case 4 :
        puts ("gain: x4 [-1.25,+1.25]V");
        break;
      case 8 :
        puts ("gain: x8 [-0.625,+0.625]V");
        break;
      case 16 :
        puts ("gain: x16 [-0.3125,+0.3125]V");
        break;
    }
    printf ("Analog input no.: %u\n",input);
  }

  fMicroWait = (seconds/fTotalSamples)*1000000.0;
  printf ("Delay between samples:%.0f ms\n",fMicroWait/1000);

  puts ("Press space+ENTER to begin or another key to exit...");

  if (getchar() != ' ') exit (EXIT_SUCCESS);
  puts ("reading data...");

  if (!(file = fopen (FileName,"w")))
  {
     printf ("error: I can't create: %s\n",FileName);
     exit (EXIT_FAILURE);
  }

  /* Set gain */
  SetGain (gain);

  i = 0;
  fCurrentSamples = 0;
  while (fCurrentSamples < fTotalSamples)
  {
    if (operation==AI)
    {
      if (!Read_AD(0,&(ADbuffer[i])))
      {
         printf ("Error reading from ADC, I/O address 0x%X correct ?\n",addr);
         exit (EXIT_FAILURE);
      }
    }
    else Read_DI (&(DIbuffer[i]));

    if (++i == BUFFER_SIZE)
    {
       WriteBuffer (i,operation);
       i = 0;
    }

    if (++fCurrentSamples < fTotalSamples)
       usleep ((unsigned long)fMicroWait);
  }

  /* save samples in buffer */
  if (i > 0)
     WriteBuffer (i,operation);

  fclose (file);

  CloseCard ();
}
/*----------------------------------------------------------------------------*/
